package com.samelement.iot_notification.appwidget.db;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import androidx.annotation.Nullable;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class WidgetReaderDbHelper extends SQLiteOpenHelper {

    private static final int DATABASE_VERSION = 7;
    private static final String DATABASE_NAME = "WidgetReader.db";

    private static final String SQL_CREATE_ENTRIES =
            "CREATE TABLE app_widget (" +
                    "widget_id TEXT," +
                    "topic TEXT," +
                    "title TEXT," +
                    "resource TEXT," +
                    "device TEXT," +
                    "datetime TEXT," +
                    "unit TEXT," +
                    "value TEXT," +
                    "color TEXT," +
                    "transparency INTEGER," +
                    "chart INTEGER," +
                    "type TEXT)";

    private static final String SQL_DELETE_ENTRIES =
            "DROP TABLE IF EXISTS app_widget";

    private static WidgetReaderDbHelper instance = null;

    public static synchronized WidgetReaderDbHelper getInstance(Context ctx) {

        // Use the application context, which will ensure that you
        // don't accidentally leak an Activity's context.
        // See this article for more information: http://bit.ly/6LRzfx
        if (instance == null) {
            instance = new WidgetReaderDbHelper(ctx.getApplicationContext());
        }
        return instance;
    }

    public WidgetReaderDbHelper(@Nullable Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL(SQL_CREATE_ENTRIES);
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        db.execSQL(SQL_DELETE_ENTRIES);
        onCreate(db);
    }

    @Override
    public void onDowngrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        onUpgrade(db, oldVersion, newVersion);
    }

    public void insert(String id, String topic, String title, String resource, String device,
                       String datetime, String unit, String value, String color,
                       int transparency, boolean isChart, String type) {
        int chart = isChart ? 1 : 0;

        SQLiteDatabase db = getWritableDatabase();
        db.execSQL("insert into app_widget(widget_id, topic, title, " +
                "resource, device, datetime, unit, value, color, transparency, chart, type) " +
                "values('" + id + "', '" + topic + "','" + title + "', '" + resource +
                "', '" + device + "', '" + datetime + "','" + unit + "','" + value + "','" +
                color + "', " + transparency + "," + chart + ", '" + type + "')");
    }

    public List<AppWidgetObject> getById(String id) {
        SQLiteDatabase db = getReadableDatabase();

        Cursor cursor = db.rawQuery("select widget_id, topic, title, resource, device, " +
                "datetime, unit, value, color, transparency, chart, type " +
                "from app_widget where widget_id='" + id + "'", null);
        List<AppWidgetObject> list = new ArrayList<>();
        try {
            while (cursor.moveToNext()) {
                AppWidgetObject appWidgetObject = new AppWidgetObject();

                appWidgetObject.setWidgetId(cursor.getString(0));
                appWidgetObject.setTopic(cursor.getString(1));
                appWidgetObject.setTitle(cursor.getString(2));
                appWidgetObject.setResource(cursor.getString(3));
                appWidgetObject.setDevice(cursor.getString(4));
                appWidgetObject.setDatetime(cursor.getString(5));
                appWidgetObject.setUnit(cursor.getString(6));
                appWidgetObject.setValue(cursor.getString(7));
                appWidgetObject.setColor(cursor.getString(8));
                appWidgetObject.setTransparency(cursor.getInt(9));
                appWidgetObject.setChart(cursor.getInt(10) == 1);
                appWidgetObject.setType(cursor.getString(11));

                list.add(appWidgetObject);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }

        return list;
    }

    public List<AppWidgetObject> getAllWhereTopic(String topic) {
        SQLiteDatabase db = getWritableDatabase();
        Cursor cursor = db.rawQuery("select widget_id, topic, title, " +
                "resource, device, datetime, unit, value, color, transparency, chart, type " +
                "from app_widget where topic='" + topic + "'", null);

        List<AppWidgetObject> list = new ArrayList<>();
        try {
            while (cursor.moveToNext()) {
                AppWidgetObject appWidgetObject = new AppWidgetObject();
                appWidgetObject.setWidgetId(cursor.getString(0));
                appWidgetObject.setTopic(cursor.getString(1));
                appWidgetObject.setTitle(cursor.getString(2));
                appWidgetObject.setResource(cursor.getString(3));
                appWidgetObject.setDevice(cursor.getString(4));
                appWidgetObject.setDatetime(cursor.getString(5));
                appWidgetObject.setUnit(cursor.getString(6));
                appWidgetObject.setValue(cursor.getString(7));

                appWidgetObject.setColor(cursor.getString(8));
                appWidgetObject.setTransparency(cursor.getInt(9));
                appWidgetObject.setChart(cursor.getInt(10) == 1);
                appWidgetObject.setType(cursor.getString(11));

                list.add(appWidgetObject);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }

        return list;
    }

    // update harus dirubah
    public List<AppWidgetObject> updateValueWhereTopic(List<AppWidgetObject> allSameTopics, String topic, String value) {
        SQLiteDatabase db = getWritableDatabase();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US);

        List<AppWidgetObject> result = new ArrayList<>();

        String currentDate = sdf.format(new Date().getTime());
        for (AppWidgetObject obj : allSameTopics) {
            String newValue = value;
            if (obj.isChart()) {
                List<String> splitted = new ArrayList<>(Arrays.asList(obj.getValue().split(";")));
                if (splitted.size() == 5) {
                    splitted.remove(0);
                }
                splitted.add(value);

                StringBuilder csvBuilder = new StringBuilder();

                String separator = ";";
                for (String str : splitted) {
                    csvBuilder.append(str);
                    csvBuilder.append(separator);
                }

                String csv = csvBuilder.toString();
                csv = csv.substring(0, csv.length() - separator.length());

                newValue = csv;
            }
            db.execSQL("update app_widget set value='" + newValue
                    + "', datetime='" + currentDate + "' where topic='" + topic
                    + "' and widget_id = '" + obj.getWidgetId() + "'");

            obj.setValue(newValue);
            obj.setDatetime(currentDate);

            result.add(obj);
        }

        return result;
    }

    public void deleteWidget(int widgetId) {
        SQLiteDatabase db = getWritableDatabase();
        db.execSQL("delete from app_widget where widget_id='" + widgetId + "'");
    }

    @Override
    public synchronized void close() {
        SQLiteDatabase db = getReadableDatabase();
        if (db != null) {
            db.close();
            super.close();
        }
    }

    @Override
    protected void finalize() throws Throwable {
        this.close();
        super.finalize();
    }
}
