package com.samelement.iot_notification.util;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Build;
import android.provider.Settings;

import java.math.BigInteger;
import java.util.Calendar;

import androidx.core.app.NotificationCompat;

import com.samelement.iot_notification.R;

public class NotificationHelper {

    public static final String SERVER_STATUS_CHANNEL_ID = "SamelCIDImportanceDef";
    public static final String SERVER_STATUS_CHANNEL_NAME = "SamelCNameImportanceDef";

    private static final String NOTIFICATION_CHANNEL_ID = "SamelNotificationCID";
    private static final String NOTIFICATION_CHANNEL_NAME = "SamelNotificationCName";

    public static final String ALARM_CHANNEL_ID = "SamelAlarmCID";
    public static final String ALARM_CHANNEL_NAME = "SamelAlarmCName";

    private Context context;
    private NotificationManager notificationManager;

    public NotificationHelper(Context context) {
        this.context = context;

        this.notificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            int importance = NotificationManager.IMPORTANCE_HIGH;
            NotificationChannel notificationChannel = new NotificationChannel(NOTIFICATION_CHANNEL_ID, NOTIFICATION_CHANNEL_NAME, importance);
            notificationChannel.enableLights(true);
            notificationChannel.setLightColor(Color.RED);
            notificationChannel.enableVibration(true);
            notificationChannel.setVibrationPattern(new long[]{100, 200, 300, 400, 500, 400, 300, 200, 400});

            if (notificationManager != null) {
                notificationManager.createNotificationChannel(notificationChannel);
            }
        }
    }

    public void createNotification(String analyticId, String title, String message, Bitmap largeIcon) {
        try {
            int notificationId = Integer.parseInt(analyticId);

            // Create an explicit intent for an Activity in your app
            Intent fullScreenIntent = new Intent(context, FlutterUtil.getMainActivityClass(context));
            fullScreenIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);

            PendingIntent fullScreenPendingIntent = PendingIntent.getActivity(context, 0,
                    fullScreenIntent, PendingIntent.FLAG_UPDATE_CURRENT);

            // create builder
            NotificationCompat.Builder builder = new NotificationCompat.Builder(context, NOTIFICATION_CHANNEL_ID);
            builder.setContentTitle(title)
                    .setContentText(message)
                    .setAutoCancel(true)
                    .setSound(Settings.System.DEFAULT_NOTIFICATION_URI)
                    .setStyle(new NotificationCompat.BigTextStyle().bigText(message))
                    .setVibrate(new long[]{1000, 1000, 1000})
                    .setPriority(NotificationCompat.PRIORITY_HIGH)
                    .setShowWhen(true)
                    .setCategory(NotificationCompat.CATEGORY_ALARM)
                    .setWhen(Calendar.getInstance().getTimeInMillis())
                    .setContentIntent(fullScreenPendingIntent);

            // set notification logo
            builder.setSmallIcon(R.drawable.ic_sam_notification_logo_trans);
            builder.setLargeIcon(largeIcon);

            if (notificationManager != null) {
                notificationManager.notify(notificationId, builder.build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
