import 'package:flutter/services.dart';

class IotNotification {
  static const MethodChannel _channel = const MethodChannel('iot_notification');

  static startService(
      String userAccessToken,
      String userRefreshToken,
      String devAccessToken,
      String devRefreshToken,
      String mqttClientIdentifier,
      String brokerAddress,
      String brokerProtocol,
      int brokerPort,
      String apiHost,
      String caFile) {
    _channel.invokeMethod("startService", {
      "user_access_token": userAccessToken,
      "user_refresh_token" : userRefreshToken,
      "dev_access_token": devAccessToken,
      "dev_refresh_token" : devRefreshToken,
      "mqtt_identifier": mqttClientIdentifier,
      "broker_address": brokerAddress,
      "broker_protocol": brokerProtocol,
      "broker_port": brokerPort,
      "api_host": apiHost,
      "ca_file": caFile
    });
  }

  static themeChanged(int themeId) {
    _channel.invokeMethod("themeChanged", {"ThemeId": themeId});
  }

  static deleteAnalytic(String jsonAnalytic) {
    _channel.invokeMethod("deleteAnalytic", {"data": jsonAnalytic});
  }

  static addDevice(String jsonDevice) {
    _channel.invokeMethod("addDevice", {"data": jsonDevice});
  }

  static deleteDevice(String jsonDevice) {
    _channel.invokeMethod("deleteDevice", {"data": jsonDevice});
  }

  static updateDevice(String jsonDevice) {
    _channel.invokeMethod("updateDevice", {"data": jsonDevice});
  }

  static addAnalytic() {
    _channel.invokeMethod("addAnalytic");
  }

  static updateAnalytic() {
    _channel.invokeMethod("updateAnalytic");
  }

  static stopService() {
    _channel.invokeMethod("stopService");
  }

  static openPowerManager() {
    _channel.invokeMethod("openPowerManager");
  }

  static ignoreBatteryOptimization() {
    _channel.invokeMethod("ignoreBatteryOptimization");
  }

  static Future<String> scan() async {
    return await _channel.invokeMethod("scan");
  }

  static Future<List<String>> getKeys() async {
    List<dynamic> keysResult = await _channel.invokeMethod("getKeys");
    List<String> keys = List<String>.from(keysResult.map((e) => e.toString()));
    return keys;
  }
}
